--- modules/stream_filter/httplive.orig.c
+++ modules/stream_filter/httplive.c
@@ -2,7 +2,7 @@
  * httplive.c: HTTP Live Streaming stream filter
  *****************************************************************************
  * Copyright (C) 2010-2011 M2X BV
- * $Id$
+ * $Id: 000a91114e9d7730d36eaacd257e610b7783ca4e $
  *
  * Author: Jean-Paul Saman <jpsaman _AT_ videolan _DOT_ org>
  *
@@ -48,6 +48,8 @@
 static int  Open (vlc_object_t *);
 static void Close(vlc_object_t *);
 
+#define HTTPLIVE_CFG_PREFIX "httplive-"
+
 vlc_module_begin()
     set_category(CAT_INPUT)
     set_subcategory(SUBCAT_INPUT_STREAM_FILTER)
@@ -650,7 +652,7 @@
     }
 
     if (hls->sequence > 0)
-        msg_Err(s, "EXT-X-MEDIA-SEQUENCE already present in playlist (new=%d, old=%d)",
+        msg_Dbg(s, "EXT-X-MEDIA-SEQUENCE already present in playlist (new=%d, old=%d)",
                     sequence, hls->sequence);
 
     hls->sequence = sequence;
@@ -965,17 +967,56 @@
 {
     stream_sys_t *p_sys = s->p_sys;
     assert(*streams);
+    int err = 0;
 
-    /* Download new playlist file from server */
-    uint8_t *buffer = NULL;
-    ssize_t len = read_M3U8_from_url(s, &p_sys->m3u8, &buffer);
-    if (len < 0)
-        return VLC_EGENERIC;
+    int count = vlc_array_count(p_sys->hls_stream);
+    /* The master list is supposed to be statefull */
+    /* Do not reload unless necesary or you might loose you server session */
+    if (count == 0)
+    {
+    	  uint8_t *buffer = NULL;
+        /* Download master playlist file from server */
+        ssize_t len = read_M3U8_from_url(s, &p_sys->m3u8, &buffer);
+        if (len < 0)
+            err = VLC_EGENERIC;
+        else
+        {
+            /* Parse HLS m3u8 content. */
+            err = parse_M3U8(s, *streams, buffer, len);
+            free(buffer);
+        }
+    }
+    else
+    {
+        /* Reload playlist info for all streams in existing master list */
+        for (int n = 0; n < count; n++)
+        {
+            uint8_t *buffer = NULL;
+            hls_stream_t *hls = vlc_array_item_at_index(p_sys->hls_stream, n);
+            if (hls == NULL)
+                continue;
+            char *psz_uri = NULL;
+            if (asprintf(&psz_uri, "%s://%s:%s@%s:%d%s", hls->url.psz_protocol,
+                 hls->url.psz_username, hls->url.psz_password,
+                 hls->url.psz_host, hls->url.i_port,hls->url.psz_path) < 0)
+                return VLC_ENOMEM;
+            hls_stream_t *hls_new = hls_New(*streams, hls->id, hls->bandwidth, psz_uri);
+            free(psz_uri);
+            ssize_t len = read_M3U8_from_url(s, &hls_new->url, &buffer);
+            if (len < 0)
+            {
+                err = VLC_EGENERIC;
+                break;
+            }
+            else
+            {
+                /* Parse HLS m3u8 content. */
+                err = parse_M3U8(s, *streams, buffer, len);
+                free(buffer);
+            }
+        }
+    }
 
-    /* Parse HLS m3u8 content. */
-    int err = parse_M3U8(s, *streams, buffer, len);
-    free(buffer);
-
     return err;
 }
 
@@ -983,7 +1024,8 @@
 static int hls_UpdatePlaylist(stream_t *s, hls_stream_t *hls_new, hls_stream_t **hls)
 {
     int count = vlc_array_count(hls_new->segments);
-
+    int newitems = 0;
+    
     msg_Info(s, "updating hls stream (program-id=%d, bandwidth=%"PRIu64") has %d segments",
              hls_new->id, hls_new->bandwidth, count);
     for (int n = 0; n < count; n++)
@@ -1000,17 +1042,17 @@
                 (p->duration != segment->duration) ||
                 (strcmp(p->url.psz_path, segment->url.psz_path) != 0))
             {
-                msg_Err(s, "existing segment found with different content - resetting");
-                msg_Err(s, "- sequence: new=%d, old=%d", p->sequence, segment->sequence);
-                msg_Err(s, "- duration: new=%d, old=%d", p->duration, segment->duration);
-                msg_Err(s, "- file: new=%s", p->url.psz_path);
-                msg_Err(s, "        old=%s", segment->url.psz_path);
+                msg_Dbg(s, "existing segment found with different content - resetting");
+                msg_Dbg(s, "- sequence: new=%d, old=%d", p->sequence, segment->sequence);
+                msg_Dbg(s, "- duration: new=%d, old=%d", p->duration, segment->duration);
+                msg_Dbg(s, "- file: new=%s", p->url.psz_path);
+                msg_Dbg(s, "        old=%s", segment->url.psz_path);
 
                 /* Resetting content */
                 segment->sequence = p->sequence;
                 segment->duration = p->duration;
                 vlc_UrlClean(&segment->url);
-                vlc_UrlParse(&segment->url, p->url.psz_buffer, 0);
+                vlc_UrlParse(&segment->url, p->url.psz_path, 0);
                 segment_Free(p);
             }
         }
@@ -1027,10 +1069,11 @@
             }
             vlc_array_append((*hls)->segments, p);
             msg_Info(s, "- segment %d appended", p->sequence);
+            newitems++;
         }
         vlc_mutex_unlock(&(*hls)->lock);
     }
-    return VLC_SUCCESS;
+    return newitems;
 
 fail_and_unlock:
     vlc_mutex_unlock(&(*hls)->lock);
@@ -1077,9 +1120,25 @@
             msg_Info(s, "new HLS stream appended (id=%d, bandwidth=%"PRIu64")",
                      hls_new->id, hls_new->bandwidth);
         }
-        else if (hls_UpdatePlaylist(s, hls_new, &hls_old) != VLC_SUCCESS)
-            msg_Info(s, "failed updating HLS stream (id=%d, bandwidth=%"PRIu64")",
-                     hls_new->id, hls_new->bandwidth);
+        else
+        {
+            msg_Dbg(s, "Updating HLS stream (id=%d, bandwidth=%"PRIu64")",
+                       hls_new->id, hls_new->bandwidth);
+            int updatecount = hls_UpdatePlaylist(s, hls_new, &hls_old);
+        	  if (updatecount > 0)
+            {
+                msg_Dbg(s, "Triggering pending downloads");
+                // Trigger pending downloads after adding segments to the list 
+                // I am not sure this is necesary when p_sys->b_live == false
+                vlc_mutex_lock(&p_sys->download.lock_wait);
+                vlc_cond_signal(&p_sys->download.wait);
+                vlc_mutex_unlock(&p_sys->download.lock_wait);
+            }
+            else if (updatecount < 0) // error
+            {  msg_Err(s, "failed updating HLS stream (id=%d, bandwidth=%"PRIu64")",
+                       hls_new->id, hls_new->bandwidth);
+            }
+        }
     }
 
     vlc_array_destroy(hls_streams);
@@ -1268,22 +1327,39 @@
 
     int canc = vlc_savecancel();
 
+    double wait_live = 0.5;
+    double wait_vod = 1;
+    double wait;
+    if (p_sys->b_live)
+        wait = wait_live;
+    else
+    	  wait = wait_vod;
+
     while (vlc_object_alive(s))
     {
-        double wait = 1;
         mtime_t now = mdate();
         if (now >= p_sys->playlist.wakeup)
         {
             /* reload the m3u8 */
             if (hls_ReloadPlaylist(s) != VLC_SUCCESS)
             {
-                /* No change in playlist, then backoff */
+            	  msg_Warn(s, "playlist reload failed, retry #%i", p_sys->playlist.tries);
                 p_sys->playlist.tries++;
-                if (p_sys->playlist.tries == 1) wait = 0.5;
-                else if (p_sys->playlist.tries == 2) wait = 1;
-                else if (p_sys->playlist.tries >= 3) wait = 3;
+                /* error reloading playlist updates */
+                /* fast retry and then backoff */
+                if (p_sys->playlist.tries == 1) wait = wait / 2;
+                else if ((p_sys->playlist.tries >= 2) && (p_sys->b_live)) wait = wait_live;
+                else if ((p_sys->playlist.tries >= 2) && (!p_sys->b_live)) wait = wait_vod;
             }
-            else p_sys->playlist.tries = 0;
+            else if (p_sys->playlist.tries > 0)
+            {
+                /* reset to default values */
+            	  p_sys->playlist.tries = 0;
+                if (p_sys->b_live)
+                    wait = wait_live;
+                else
+                    wait = wait_vod;
+            }
 
             hls_stream_t *hls = hls_Get(p_sys->hls_stream, p_sys->download.stream);
             assert(hls);
